package org.vishia.gral.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.events.DragDetectEvent;
import org.eclipse.swt.events.DragDetectListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.vishia.byteData.VariableAccessWithIdx;
import org.vishia.gral.base.GralButton.GraphicImplAccess;
import org.vishia.gral.base.GralKeyListener;
import org.vishia.gral.base.GralMouseWidgetAction_ifc;
import org.vishia.gral.base.GralPos;
import org.vishia.gral.base.GralWidget;
import org.vishia.gral.base.GralMng;
import org.vishia.gral.base.GralTextField;
import org.vishia.gral.ifc.GralColor;
import org.vishia.gral.ifc.GralFont;
import org.vishia.gral.ifc.GralRectangle;
import org.vishia.gral.ifc.GralTextFieldUser_ifc;
import org.vishia.gral.ifc.GralUserAction;
import org.vishia.gral.ifc.GralWidget_ifc;
import org.vishia.util.KeyCode;

public class SwtTextFieldWrapper extends GralTextField.GraphicImplAccess
{

  /**Version, history and license.
   * <ul>
   * <li>2015-05-04 Hartmut new: {@link #setBorderWidth(int)} to show the text field with a border. That is not a property
   *   of an SWT Text, therefore a new {@link #paintListener} was added to draw the border.
   * <li>2013-12-22 Hartmut chg: Now {@link GralTextField} uses the new concept of instantiation: It is not
   *   the super class of the implementation class. But it provides {@link GralTextField.GraphicImplAccess}
   *   as the super class. 
   * <li>2012-06-30 Hartmut new actionChange called on typing inside a field.
   * <li>2012-06-30 Hartmut new {@link #swtKeyListener}. The [Enter] key will be send to the User
   *   in opposite to {@link SwtTextBox}.
   * <li>2012-06-08 Hartmut chg: {@link #repaintGthread()} does not do anything if the textFieldSwt is removed 
   *   because the widget was removed. Prevent null-Pointer exception.
   * <li>2012-04-10 Hartmut chg: A key listener, only for experience
   * <li>2012-03-17 Hartmut bugfix: adjustment of prompt for top prompt
   * <li>2012-03-10 Hartmut chg: Minor for top-level prompt.
   * <li>2011-06-00 Hartmut creation
   * </ul>
   * 
   * <b>Copyright/Copyleft</b>:
   * For this source the LGPL Lesser General Public License,
   * published by the Free Software Foundation is valid.
   * It means:
   * <ol>
   * <li> You can use this source without any restriction for any desired purpose.
   * <li> You can redistribute copies of this source to everybody.
   * <li> Every user of this source, also the user of redistribute copies
   *    with or without payment, must accept this license for further using.
   * <li> But the LPGL is not appropriate for a whole software product,
   *    if this source is only a part of them. It means, the user
   *    must publish this part of source,
   *    but doesn't need to publish the whole source of the own product.
   * <li> You can study and modify (improve) this source
   *    for own using or for redistribution, but you have to license the
   *    modified sources likewise under this LGPL Lesser General Public License.
   *    You mustn't delete this Copyright/Copyleft inscription in this source file.
   * </ol>
   * If you intent to use this source without publishing its usage, you can get
   * a second license subscribing a special contract with the author. 
   * 
   * @author Hartmut Schorrig = hartmut.schorrig@vishia.de
   * 
   * 
   */
  //@SuppressWarnings("hiding")
  public static final String version = "2015-09-12";
  
  /**It contains the association to the swt widget (Control) and the {@link SwtMng}
   * and implements some methods of {@link GralWidgImpl_ifc} which are delegate from this.
   */
  private final SwtWidgetHelper swtWidgHelper;
  
  protected Text textFieldSwt;
  
  /**A possible prompt for the text field or null. */
  //Label promptSwt;
  SwtTransparentLabel promptSwt;
  //Text promptSwt;
  
  private DropTarget drop;
  
  
  private SwtTextFieldWrapper(GralTextField widgg, SwtMng mng)
  {
    widgg.super(widgg); //NOTE: superclass is a non static inner class of GralTextField. 

    Composite panelSwt = mng.getCurrentPanel();
    //in ctor: setPanelMng(mng);
    //Text widgetSwt;
    //
    int textProperties = SWT.SINGLE;
    if(isPasswordField()){ 
      textProperties |= SWT.PASSWORD; 
    }
    if(posPrompt !=null) {
      final Font promptFont;
      GralRectangle boundsPrompt;
      float heightPrompt = posPrompt.height();
      promptFont = mng.propertiesGuiSwt.getTextFontSwt(heightPrompt, GralFont.typeSansSerif, GralFont.styleNormal); //.smallPromptFont;
      //boundsPrompt = mng.calcWidgetPosAndSize(posPrompt, boundsAll.dx, boundsAll.dy, 10,100);
      //boundsField = mng.calcWidgetPosAndSize(posField, boundsAll.dx, boundsAll.dy, 10,100);
      promptSwt = new SwtTransparentLabel(panelSwt, SWT.TRANSPARENT);
      promptSwt.setFont(promptFont);
      promptSwt.setText(prompt());
      promptSwt.setBackground(null);
      Point promptSize = promptSwt.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
      boundsPrompt = mng.calcWidgetPosAndSizeSwt(posPrompt, promptSwt.getParent(), 10,100);
      if(promptSize.x > boundsPrompt.dx){
        boundsPrompt.dx = promptSize.x;  //use the longer value, if the prompt text is longer as the field.
      }
      promptSwt.setBounds(boundsPrompt.x, boundsPrompt.y, boundsPrompt.dx, boundsPrompt.dy+1);
    }
    textFieldSwt =  new Text(panelSwt, textProperties);
    mng.setPosAndSizeSwt(posField,textFieldSwt, 800, 600);
    //textFieldSwt.setBounds(boundsField.x, boundsField.y, boundsField.dx, boundsField.dy);
    textFieldSwt.setFont(mng.propertiesGuiSwt.stdInputFont);
    textFieldSwt.setEditable(widgg.isEditable());
    textFieldSwt.setBackground(mng.propertiesGuiSwt.colorSwt(GralColor.getColor("wh")));
    KeyListener swtKeyListener = new TextFieldKeyListener(mng.mng._impl.gralKeyListener);
    textFieldSwt.addKeyListener(swtKeyListener);
    textFieldSwt.setMenu(null);  //default: no contextMenu, use GralMenu?
    
    Listener[] oldMouseListener = textFieldSwt.getListeners(SWT.MouseDown);
    for(Listener lst: oldMouseListener){
      textFieldSwt.removeListener(SWT.MouseDown, lst);
    }
    textFieldSwt.addMouseListener(mng.mouseClickForInfo);
    textFieldSwt.addFocusListener(mng.focusListener);
    if(widgg.isEditable()){
      TextFieldModifyListener modifyListener = new TextFieldModifyListener();
      textFieldSwt.addModifyListener(modifyListener);
      TextFieldFocusListener focusListener = new TextFieldFocusListener(mng);
      textFieldSwt.addFocusListener(focusListener);
    } else {
      
    }
    if(prompt() != null && promptStylePosition() !=null && promptStylePosition().startsWith("r")){
      Rectangle swtField = textFieldSwt.getBounds();
      Rectangle swtPrompt = new Rectangle(swtField.x + swtField.width, swtField.y, 0, swtField.height);
      float hight = widgg.pos().height();
      final Font promptFont;
      if(hight <2.0){
        promptFont = mng.propertiesGuiSwt.smallPromptFont;  
      } else { 
        promptFont = mng.propertiesGuiSwt.stdInputFont;  
      }
      promptSwt = new SwtTransparentLabel(mng.getCurrentPanel(), SWT.TRANSPARENT);
      promptSwt.setFont(promptFont);
      promptSwt.setText(prompt());
      Point promptSize = promptSwt.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
      swtPrompt.width = promptSize.x;
      promptSwt.setBounds(swtPrompt);
      
      
    }
    //
    textFieldSwt.setData(this);
    textFieldSwt.addPaintListener(paintListener);
    if(!widgg.isEditable()){
      mng.mng.registerShowField(widgg);
    }
    super.wdgimpl = swtWidgHelper = new SwtWidgetHelper(textFieldSwt, mng);

    mng.mng.registerWidget(widgg);
    
  }

  
  
  /**Creates a SwtTextField. It is package private, only called from the {@link SwtMng}.
   * @param widgg
   * <br>
   * <b>Prompting</b>: The parameter promptStylePosition determines where a prompt is showing.
   * <ul>
   * <li>"t": prompt above, calculates inside position
   * <li>"r": prompt right, calculates outside position.
   * </ul>
   * @param name The name to register it.
   * @param editable false then show field
   * @param prompt maybe null, propmt text
   * @param promptStylePosition maybe null, prompt position
   * @param mng
   */
  static void createTextField(GralTextField widgg, GralMng mng){
    SwtTextFieldWrapper widgswt = new SwtTextFieldWrapper(widgg, (SwtMng)mng.impl);
  }
  
  
  @Override public GralRectangle getPixelPositionSize(){ return swtWidgHelper.getPixelPositionSize(); }



  
  @Override
  protected void setDropEnable(int dropType)
  {
    new SwtDropListener(dropType, textFieldSwt); //associated with textFieldSwt.
  }
  
  
  @Override
  protected void setDragEnable(int dragType)
  {
    new SwtDragListener(dragType, textFieldSwt); //associated with textFieldSwt.
  }
  
  /*
  @Override public void setSelection(String how){
    if(how.equals("|..<")){
      String sText = textFieldSwt.getText();
      int zChars = sText.length();
      int pos0 = 0; //zChars - 20;
      if(pos0 < 0){ pos0 = 0; }
      textFieldSwt.setSelection(pos0, zChars);
    }
  }
  */
  

  
  
  @Override public void repaintGthread(){
    int catastrophicalCount = 0;
    int chg;
    if(textFieldSwt !=null){ //do nothing if the graphic implementation widget is removed.
      GralWidget.DynamicData dyda = dyda();
      while( (chg = getChanged()) !=0){ //widgg.dyda.whatIsChanged.get();
        if(++catastrophicalCount > 10000) 
          throw new RuntimeException("atomic failed");
        if((chg & chgText) !=0 && dyda.displayedText !=null){ 
          textFieldSwt.setText(dyda.displayedText);
          final int selectionStart, selectionEnd;
          final int zText = dyda.displayedText.length();
          if(caretPos() <0){
            selectionEnd = dyda.displayedText.length(); selectionStart = selectionEnd; // -1;
          }
          else if(caretPos() >0){
            selectionEnd = caretPos() > zText ? zText : caretPos();
            selectionStart = selectionEnd; // -1;
          } else {
            assert(caretPos() ==0);
            selectionEnd = selectionStart =-1;  //dont call
          }
          if(selectionStart >=0){
            textFieldSwt.setSelection(selectionStart, selectionEnd);
          }
        }
        if((chg & chgColorText)!=0){
          SwtProperties props = swtWidgHelper.mng.propertiesGuiSwt;
          if(dyda.textColor !=null){
            textFieldSwt.setForeground(props.colorSwt(dyda.textColor));
          }
          if(dyda.backColor !=null){
            textFieldSwt.setBackground(props.colorSwt(dyda.backColor));
          }
          if(dyda.textFont !=null){
            textFieldSwt.setFont(props.fontSwt(dyda.textFont));
          }
        }
        if((chg & chgEditable) !=0){ 
          textFieldSwt.setEditable(widgg.isEditable());
        }
        if((chg & chgCursor) !=0){ 
          textFieldSwt.setSelection(caretPos());
        }
        if((chg & chgPrompt) !=0){ 
          promptSwt.setText(this.prompt());
          promptSwt.redraw();
        }
        if((chg & chgVisible) !=0){ 
          textFieldSwt.setVisible(true);
        }
        if((chg & chgInvisible) !=0){ 
          textFieldSwt.setVisible(false);
        }
        if((chg & chgColorText) !=0){ 
          textFieldSwt.setForeground(swtWidgHelper.mng.getColorImpl(dyda().textColor)); }
        if((chg & chgColorBack) !=0){ 
          textFieldSwt.setBackground(swtWidgHelper.mng.getColorImpl(dyda().backColor)); }
        textFieldSwt.redraw();
        //textFieldSwt.
        acknChanged(chg);
      }
    }
  }

  

  @Override public Text getWidgetImplementation()
  { return textFieldSwt;
  }

  


  
  @Override public boolean setFocusGThread()
  { return SwtWidgetHelper.setFocusOfTabSwt(textFieldSwt);
  }

  @Override public void setVisibleGThread(boolean bVisible) { super.setVisibleState(bVisible); swtWidgHelper.setVisibleGThread(bVisible); }


  @Override public void removeWidgetImplementation()
  {
    if(textFieldSwt !=null){
      textFieldSwt.dispose();
    } else {
      stop();
    }
    textFieldSwt = null;
    if(promptSwt !=null){
      promptSwt.dispose();
      promptSwt = null;
    }
  }

  @Override public void setBoundsPixel(int x, int y, int dx, int dy)
  { textFieldSwt.setBounds(x,y,dx,dy);
  }
  
  
  protected void textFieldFocusGained(){
    //- done in GralMng.GralMngFocusListener! super.focusGained();  //set HtmlHelp etc.
    GralWidget_ifc.ActionChange action = getActionChange(GralWidget_ifc.ActionChangeWhen.onFocusGained); 
    if(action !=null){
      Object[] args = action.args();
      if(args == null){ action.action().exec(KeyCode.focusGained, widgg, dyda().displayedText); }
      else { action.action().exec(KeyCode.focusGained, widgg, args, dyda().displayedText); }
    }
    if(dyda().displayedText !=null){
      textFieldSwt.setText(dyda().displayedText);
    }
  }
  
  
  protected void textFieldFocusLost(){
    String text2 = textFieldSwt.getText();
    //There was a problem. Because of TextFieldModifyListener the field is already set. Newly read of getText() gets the old text. Bug of windows?
    //dyda().displayedText = text2;  //transfer the current text
    caretPos(textFieldSwt.getCaretPosition());
    //TODO only invoke on changed content.
    GralWidget_ifc.ActionChange action = getActionChange(GralWidget_ifc.ActionChangeWhen.onChangeAndFocusLost); 
    if(action !=null){
      Object[] args = action.args();
      if(args == null){ action.action().exec(KeyCode.focusLost, widgg, text2); }
      else { action.action().exec(KeyCode.focusLost, widgg, args, text2); }
    }
  }
  
  protected void paintWidget(Text swt, PaintEvent e){
    GC gc = e.gc;
    //gc.d
    int borderwidth = super.borderwidth();
    if(borderwidth >0) {
      GralTextField widg = (GralTextField)super.widgg;
      Rectangle dim = swt.getBounds();
      gc.setLineWidth(borderwidth);
      Color colorLine = swtWidgHelper.mng.getColorImpl(dyda().lineColor);
      gc.setForeground(colorLine);
      //gc.drawLine(0, 0, dim.width, dim.height);  //test of coordinates
      Rectangle rect = new Rectangle(0,0,dim.width, dim.height);  //the rect which should drawn counts from (0,0)
      gc.drawRectangle(rect);
    }
  } 
  
  PaintListener paintListener = new PaintListener(){
    @Override public void paintControl(PaintEvent e) {
      SwtTextFieldWrapper.this.paintWidget((Text)swtWidgHelper.widgetSwt, e);
    }
  };
  

  
  /**For edit able fields.
   */
  private class TextFieldFocusListener extends SwtMng.SwtMngFocusListener
  {
    
    TextFieldFocusListener(SwtMng mng){
      mng.super(mng.mng);
    }

    @Override public void focusLost(FocusEvent ev){
      SwtTextFieldWrapper.this.textFieldFocusLost();
      setTouched(false);   //assumes that the changed field content is processed with routine above. Can be automaticly overwritten newly be application.
      super.focusLost(ev);
    }

    
    @Override public void focusGained(FocusEvent ev)
    { SwtTextFieldWrapper.this.textFieldFocusGained();
      super.focusGained(ev);
    }
  }
  

  
  protected class TextFieldModifyListener implements ModifyListener{
    @Override public void modifyText(ModifyEvent ev) {
      String text = textFieldSwt.getText();
      GralWidget.DynamicData dyda = SwtTextFieldWrapper.super.dyda();
      if(! text.equals(dyda.displayedText)) {
        dyda.displayedText = text;
        setTouched(true);
        //System.out.println("actionText");
        //SwtTextFieldWrapper.super.caretPos = textFieldSwt.getCaretPosition();
        GralWidget_ifc.ActionChange action = getActionChange(GralWidget_ifc.ActionChangeWhen.onAnyChgContent); 
        if(action !=null){
          Object[] args = action.args();
          if(args == null){ action.action().exec(KeyCode.valueChanged, widgg, text); }
          else { action.action().exec(KeyCode.valueChanged, widgg, args, text); }
        }
      }
      //if(dyda.displayedText !=null){
        //textFieldSwt.setText(dyda.displayedText);
      //}
    }
    
  };
  
 
  
  protected class TextFieldKeyListener extends SwtKeyListener
  {

    public TextFieldKeyListener(GralKeyListener keyAction)
    { super(keyAction);
    }

    @Override public final boolean specialKeysOfWidgetType(int key, GralWidget_ifc widgg, Object widgImpl){ 
      boolean bDone = true;
      boolean bEditable = widgg.isEditable();
      if(bEditable && KeyCode.isWritingKey(key)){
        setTextChanged();
      }
      if(bEditable && key != KeyCode.enter && KeyCode.isWritingOrTextNavigationKey(key)){
        bDone = true;
        setTouched(true);
      } else {
        boolean bUserOk;
        GralTextFieldUser_ifc user = user();
        if(user !=null){
          Point selection = textFieldSwt.getSelection();
          bUserOk = user.userKey(key
              , textFieldSwt.getText()
              , textFieldSwt.getCaretPosition()
              , selection.x, selection.y);
        } else bUserOk = false;
        if(!bUserOk ){
          switch(key){
            case KeyCode.ctrl + 'a': { 
              textFieldSwt.selectAll();
            } break;
            default: bDone = false;
          }
        }
      }
      return bDone; 
    }
  };
  
  void stop(){}






}
